@@ -341,14 +341,12 @@ who\"
             f.write("[core]\n    filemode = true\n")
             temp_path = f.name
 
-        try:
-            # Test with pathlib.Path
-            path_obj = Path(temp_path)
-            cf = ConfigFile.from_path(path_obj)
-            self.assertEqual(cf.get((b"core",), b"filemode"), b"true")
-        finally:
-            # Clean up
-            os.unlink(temp_path)
+        self.addCleanup(os.unlink, temp_path)
+
+        # Test with pathlib.Path
+        path_obj = Path(temp_path)
+        cf = ConfigFile.from_path(path_obj)
+        self.assertEqual(cf.get((b"core",), b"filemode"), b"true")
 
     def test_write_to_path_pathlib(self) -> None:
         import tempfile
@@ -1102,8 +1100,11 @@ class CaseInsensitiveConfigTests(TestCase):
         self.assertEqual("value", config.get(("core",)))
         self.assertEqual("value", config.get(("CORE",)))
         self.assertEqual("default", config.get(("missing",), "default"))
-        # Test SENTINEL behavior
-        result = config.get(("missing",))
+        # Test default_factory behavior
+        config_with_factory = CaseInsensitiveOrderedMultiDict(
+            default_factory=CaseInsensitiveOrderedMultiDict
+        )
+        result = config_with_factory.get(("missing",))
         self.assertIsInstance(result, CaseInsensitiveOrderedMultiDict)
         self.assertEqual(0, len(result))
 
