@@ -204,18 +204,16 @@ class IgnoreFilterTests(TestCase):
             f.write("*.pyc\n__pycache__/\n")
             temp_path = f.name
 
-        try:
-            # Test with pathlib.Path
-            path_obj = Path(temp_path)
-            ignore_filter = IgnoreFilter.from_path(path_obj)
+        self.addCleanup(os.unlink, temp_path)
 
-            # Test that it loaded the patterns correctly
-            self.assertTrue(ignore_filter.is_ignored("test.pyc"))
-            self.assertTrue(ignore_filter.is_ignored("__pycache__/"))
-            self.assertFalse(ignore_filter.is_ignored("test.py"))
-        finally:
-            # Clean up
-            os.unlink(temp_path)
+        # Test with pathlib.Path
+        path_obj = Path(temp_path)
+        ignore_filter = IgnoreFilter.from_path(path_obj)
+
+        # Test that it loaded the patterns correctly
+        self.assertTrue(ignore_filter.is_ignored("test.pyc"))
+        self.assertTrue(ignore_filter.is_ignored("__pycache__/"))
+        self.assertFalse(ignore_filter.is_ignored("test.py"))
 
 
 class IgnoreFilterStackTests(TestCase):
